/**
*******************************************************************************
* Copyright (C) 1996-2001, International Business Machines Corporation and    *
* others. All Rights Reserved.                                                *
*******************************************************************************
*
* $Source: 
*  /usr/cvs/icu4j/icu4j/src/com/ibm/icu/test/text/CollatorJNIPerformanceTest.java,v $ 
* $Date: 2001/03/12 20:35:56 $ 
* $Revision: 1.1 $
*
*******************************************************************************
*/

package com.ibm.icu4jni.test.text;

import java.util.Locale;
import com.ibm.icu4jni.text.Collator;
import com.ibm.icu4jni.text.CollationAttribute;
import com.ibm.icu4jni.text.RuleBasedCollator;
import com.ibm.icu4jni.text.CollationKey;

/**
* Performance testing class for Collator
* @author Syn Wee Quek
* @since jan 23 2001
*/
public final class CollatorMarkPerformanceTest 
{ 
  
  // constructor ===================================================
  
  /**
  * Constructor
  */
  public CollatorMarkPerformanceTest()
  {
  }
  
  // public methods ================================================

  /**
  * Sortkey retrieval performance testing
  * @param locale for processing
  * @param source test string
  * @param target test string
  * @return time taken for loop
  */
  public long testSortKey(Locale locale, String source, String target) 
                                                           throws Exception
  {
    System.gc();
    RuleBasedCollator collator = 
                        (RuleBasedCollator)Collator.getInstance(locale);
    collator.setStrength(CollationAttribute.VALUE_TERTIARY);
    
    // initial startup cost
    CollationKey src = collator.getCollationKey(source);
    CollationKey tgt = collator.getCollationKey(target);
    src.compareTo(tgt);
    
    long start = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
      src = collator.getCollationKey(source);
      tgt = collator.getCollationKey(target);
      src.compareTo(tgt);
    }
   
    long end = System.currentTimeMillis();
    
    long loopstart = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
      src = null;
      tgt = null;
    }
  
    long loopend = System.currentTimeMillis();
    
    collator = null;
    return (end - start) - (loopend - loopstart);
  }
  
  /**
  * Sortkey retrieval performance testing
  * @param locale for processing
  * @param source test string
  * @param target test string
  * @return time taken for loop
  */
  public long testJDKSortKey(Locale locale, String source, String target) 
                                                           throws Exception
  {
    System.gc();
    java.text.Collator collator = java.text.Collator.getInstance(locale);
    collator.setStrength(java.text.Collator.TERTIARY);
    
    // initial startup cost
    java.text.CollationKey src = collator.getCollationKey(source);
    java.text.CollationKey tgt = collator.getCollationKey(target);
    src.compareTo(tgt);
    
    long start = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
      src = collator.getCollationKey(source);
      tgt = collator.getCollationKey(target);
      src.compareTo(tgt);
    }
   
    long end = System.currentTimeMillis();
    
    long loopstart = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
      src = null;
      tgt = null;
    }
  
    long loopend = System.currentTimeMillis();
    
    collator = null;
    return (end - start) - (loopend - loopstart);
  }
  
  /**
  * Sortkey retrieval performance testing
  * @param locale for processing
  * @param source test string
  * @param target test string
  * @return time taken for loop
  */
  public long testCompare(Locale locale, String source, String target) 
                                                           throws Exception
  {
    System.gc();
    RuleBasedCollator collator = 
                        (RuleBasedCollator)Collator.getInstance(locale);
    collator.setStrength(CollationAttribute.VALUE_TERTIARY);
    
    collator.compare(source, target);
    
    long start = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
      collator.compare(source, target);
    }
   
    long end = System.currentTimeMillis();
    
    long loopstart = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
    }
  
    long loopend = System.currentTimeMillis();
    
    collator = null;
    return (end - start) - (loopend - loopstart);
  }
  
  /**
  * Sortkey retrieval performance testing
  * @param locale for processing
  * @param source test string
  * @param target test string
  * @return time taken for loop
  */
  public long testJDKCompare(Locale locale, String source, String target) 
                                                           throws Exception
  {
    System.gc();
    java.text.Collator collator = java.text.Collator.getInstance(locale);
    collator.setStrength(java.text.Collator.TERTIARY);
    
    // initial startup cost
    collator.compare(source, target);
    
    long start = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
      collator.compare(source, target);
    }
   
    long end = System.currentTimeMillis();
    
    long loopstart = System.currentTimeMillis();
    for (int i = 0; i < LOOP_COUNT_; i ++)
    {
    }
  
    long loopend = System.currentTimeMillis();
    
    collator = null;
    return (end - start) - (loopend - loopstart);
  }
  
  /**
  * Main method for running test
  * @param arg argument
  */
  public static void main(String[] arg)
  {
    Locale locale[] = {Locale.US, Locale.FRANCE};
    
    long time = 0,
         jdktime = 0;
    CollatorMarkPerformanceTest test = new CollatorMarkPerformanceTest();
    
    try
    {
      System.out.println("\nSame Strings");
      System.out.println("US        US JDK    FRANCE    FRANCE JDK");
      // doing a first round to remove any possibility of startup cost
      for (int j = 0; j < locale.length; j ++)
      {
        time = 0;
        jdktime = 0;
        for (int i = 0; i < CONSISTENCY_COUNT_; i ++)
        {
          time += test.testSortKey(locale[j], DATA_SAME_[0], DATA_SAME_[1]);
          jdktime += test.testJDKSortKey(locale[j], DATA_SAME_[0], 
                                     DATA_SAME_[1]);
        }
        System.out.print((time / CONSISTENCY_COUNT_) + "      ");
        System.out.print((jdktime / CONSISTENCY_COUNT_) + "      ");
      }
      
      System.out.println("\nCase Different Strings");
      System.out.println("US        US JDK    FRANCE    FRANCE JDK");
      // doing a first round to remove any possibility of startup cost
      for (int j = 0; j < locale.length; j ++)
      {
        time = 0;
        jdktime = 0;
        for (int i = 0; i < CONSISTENCY_COUNT_; i ++)
        {
          time += test.testSortKey(locale[j], DATA_CASE_DIFF_[0], 
                                  DATA_CASE_DIFF_[1]);
          jdktime += test.testJDKSortKey(locale[j], DATA_CASE_DIFF_[0], 
                                     DATA_CASE_DIFF_[1]);
        }
        System.out.print((time / CONSISTENCY_COUNT_) + "      ");
        System.out.print((jdktime / CONSISTENCY_COUNT_) + "      ");
      }
      
      System.out.println("\nDifferent Strings");
      System.out.println("US        US JDK    FRANCE    FRANCE JDK");
      // doing a first round to remove any possibility of startup cost
      // doing a first round to remove any possibility of startup cost
      for (int j = 0; j < locale.length; j ++)
      {
        time = 0;
        jdktime = 0;
        for (int i = 0; i < CONSISTENCY_COUNT_; i ++)
        {
          time += test.testSortKey(locale[j], DATA_DIFF_[0], DATA_DIFF_[1]);
          jdktime += test.testJDKSortKey(locale[j], DATA_DIFF_[0], 
                                     DATA_DIFF_[1]);
        }
        System.out.print((time / CONSISTENCY_COUNT_) + "      ");
        System.out.print((jdktime / CONSISTENCY_COUNT_) + "      ");
      }
      
      System.out.println("\nSame Strings Simple Compare");
      System.out.println("US        US JDK    FRANCE    FRANCE JDK");
      // doing a first round to remove any possibility of startup cost
      // doing a first round to remove any possibility of startup cost
      for (int j = 0; j < locale.length; j ++)
      {
        time = 0;
        jdktime = 0;
        for (int i = 0; i < CONSISTENCY_COUNT_; i ++)
        {
          time += test.testCompare(locale[j], DATA_SAME_[0], DATA_SAME_[1]);
          jdktime += test.testJDKCompare(locale[j], DATA_SAME_[0], 
                                     DATA_SAME_[1]);
        }
        System.out.print((time / CONSISTENCY_COUNT_) + "      ");
        System.out.print((jdktime / CONSISTENCY_COUNT_) + "      ");
      }
      
      System.out.println("\nCase Different Strings");
      System.out.println("US        US JDK    FRANCE    FRANCE JDK");
      // doing a first round to remove any possibility of startup cost
      // doing a first round to remove any possibility of startup cost
      for (int j = 0; j < locale.length; j ++)
      {
        time = 0;
        jdktime = 0;
        for (int i = 0; i < CONSISTENCY_COUNT_; i ++)
        {
          time += test.testCompare(locale[j], DATA_CASE_DIFF_[0], 
                                  DATA_CASE_DIFF_[1]);
          jdktime += test.testJDKCompare(locale[j], DATA_CASE_DIFF_[0], 
                                     DATA_CASE_DIFF_[1]);
        }
        System.out.print((time / CONSISTENCY_COUNT_) + "      ");
        System.out.print((jdktime / CONSISTENCY_COUNT_) + "      ");
      }
      
      System.out.println("\nDifferent Strings");
      System.out.println("US        US JDK    FRANCE    FRANCE JDK");
      // doing a first round to remove any possibility of startup cost
      // doing a first round to remove any possibility of startup cost
      for (int j = 0; j < locale.length; j ++)
      {
        time = 0;
        jdktime = 0;
        for (int i = 0; i < CONSISTENCY_COUNT_; i ++)
        {
          time += test.testCompare(locale[j], DATA_DIFF_[0], DATA_DIFF_[1]);
          jdktime += test.testJDKCompare(locale[j], DATA_DIFF_[0], 
                                     DATA_DIFF_[1]);
        }
        System.out.print((time / CONSISTENCY_COUNT_) + "      ");
        System.out.print((jdktime / CONSISTENCY_COUNT_) + "      ");
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }
  
  // private variables =============================================
  
  /**
  * RuleBasedCollator for performance testing
  */
  private Collator m_collator_;
  
  /**
  * Number times for looping
  */
  private static final int LOOP_COUNT_ = 1000000;
  
  /**
  * Number of times for running the loops to determine consistency
  */
  private static final int CONSISTENCY_COUNT_ = 10;
  
  /**
  * Test string using Mark Davis
  */
  private static final String DATA_SAME_[] = {"Mark Davis",
                                              "Mark Davis"};
   
  /**
  * Test string using Mark Davis with case difference
  */
  private static final String DATA_CASE_DIFF_[] = {"Mark Davis",
                                                   "mark davis"};
                                                 
  /**
  * Test string using Mark Davis and Syn Wee
  */
  private static final String DATA_DIFF_[] = {"Mark Davis",
                                              "Syn Wee Quek"};                                                 
}

